#include <stdlib.h>
#include <stdio.h>

struct node
{
  int data;
  struct node *next;
};

/* Build the list {1, 2, 3} in the heap and store
   its head pointer in a local stack variable.
   Returns the head pointer to the caller. */

struct node *
BuildOneTwoThree ()
{
  struct node *head = NULL;
  struct node *second = NULL;
  struct node *third = NULL;
  head = malloc (sizeof (struct node));	/* allocate 3 nodes in the heap */
  second = malloc (sizeof (struct node));
  third = malloc (sizeof (struct node));
  head->data = 1;		/* setup first node */
  head->next = second;
  second->data = 2;		/* setup second node */
  second->next = third;
  third->data = 3;		/* setup third link */
  third->next = NULL;
/* At this point, the linked list referenced by "head"
   matches the list in the drawing. */
  return head;
}

/* Given a linked list head pointer, compute
   and return the number of nodes in the list. */

int
Length (struct node *head)
{
  struct node *current = head;
  int count = 0;
  while (current != NULL)
    {
      count++;
      current = current->next;
    }
  return count;
}

/* Given a linked list head pointer, display
   all numbers stored in the list. */

void
Display (struct node *head)
{
  struct node *current = head;
  while (current != NULL)
    {
      printf ("%d ", current->data);
      current = current->next;
    }
  printf ("\n");
}


/* Takes a list and a data value.
   Creates a new link with the given data and pushes
   it onto the front of the list.
   The list is not passed in by its head pointer.
   Instead the list is passed in as a "reference" pointer
   to the head pointer -- this allows us
   to modify the caller's memory. */

void
Push (struct node **headPtr, int data)
{
  struct node *newNode = malloc (sizeof (struct node));
  newNode->data = data;
  newNode->next = *headPtr;	/* The '*' to dereferences back to the real head */
  *headPtr = newNode;		/* ditto */
}

void
PushTest ()
{
  struct node *head = BuildOneTwoThree ();
  Push (&head, 0);		/* note the & */
  Push (&head, 13);
/* head is now the list {13, 0, 1, 2, 3} */
}


void
FreeList (struct node *head)
{
  while (head)
    {
      struct node *t = head->next;
      free (head);
      head = t;
    };
}


void
AppendNode (struct node **headRef, int num)
{
  struct node *current = *headRef;
  struct node *newNode;
  newNode = malloc (sizeof (struct node));
  newNode->data = num;
  newNode->next = NULL;
  /* special case for length 0 */
  if (current == NULL)
    {
      *headRef = newNode;
    }
  else
    {
      /* Locate the last node */
      while (current->next != NULL)
	{
	  current = current->next;
	}
      current->next = newNode;
    }
}


struct node *
CopyList (struct node *src)
{
  struct node *head = NULL;
  struct node **dst = &head;
  while (src)
    {
      *dst = malloc (sizeof (struct node));
      (*dst)->data = src->data;
      (*dst)->next = NULL;
      src = src->next;
      dst = &((*dst)->next);
    }
  return head;
}



int
main ()
{
  struct node *list = BuildOneTwoThree ();
  struct node *copy;
  printf ("List lenght = %d\n", Length (list));
  Display (list);
  AppendNode (&list, 5);
  Display (list);
  copy = CopyList (list);
  FreeList (list);
  Display (copy);
  FreeList (copy);
  return 0;
}
